﻿/*
 * ASP.NET Validation Controls for MVC (using WebUIValidation.js)
 *
 * Copyright (c) 2009 Guramrit Singh Dhillon
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

using System.Globalization;
using System.Web.UI.WebControls;

namespace Classic.Mvc.Web.UI.Validators
{
    public abstract class BaseCompareValidator
        : BaseValidator
    {
        internal BaseCompareValidator(string id, string controlToValidate, object htmlAttributes)
            : base(id, controlToValidate, htmlAttributes)
        {
        }

        public ValidationDataType Type { get; set; }

        protected override void AddScriptAttributes()
        {
            base.AddScriptAttributes();
            if (Type != ValidationDataType.String)
                RegisterScriptAttribute("type", Type.ToString());
            NumberFormatInfo numberFormat;
            switch (Type)
            {
                case ValidationDataType.Currency:
                    numberFormat = CultureInfo.CurrentCulture.NumberFormat;
                    RegisterScriptAttribute("decimalchar", numberFormat.CurrencyDecimalSeparator);
                    RegisterScriptAttribute("groupchar", numberFormat.CurrencyGroupSeparator);
                    RegisterScriptAttribute("digits", numberFormat.CurrencyDecimalDigits.ToString());
                    RegisterScriptAttribute("groupsize", numberFormat.CurrencyGroupSizes[0].ToString());
                    break;
                case ValidationDataType.Date:
                    var dateTimeFormat = CultureInfo.CurrentCulture.DateTimeFormat;
                    var pattern = dateTimeFormat.ShortDatePattern;
                    var dateorder = (pattern.StartsWith("y", true, CultureInfo.InvariantCulture)
                                         ? "ymd"
                                         : (pattern.StartsWith("m", true, CultureInfo.InvariantCulture)
                                                ? "mdy"
                                                : "dmy"));
                    RegisterScriptAttribute("dateorder", dateorder);
                    RegisterScriptAttribute("cutoffyear", dateTimeFormat.Calendar.TwoDigitYearMax.ToString());
                    RegisterScriptAttribute("century", dateTimeFormat.Calendar.ToFourDigitYear(0).ToString());
                    break;
                case ValidationDataType.Double:
                    numberFormat = CultureInfo.CurrentCulture.NumberFormat;
                    RegisterScriptAttribute("decimalchar", numberFormat.CurrencyDecimalSeparator);
                    break;
            }
        }
    }
}